/*
 *  Copyright (C) 1999, 2000  Heikki Tauriainen <Heikki.Tauriainen@hut.fi>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

%{

#include "NeverClaimParser.h"

NeverClaimParser::NeverClaimAutomaton* automaton;



/******************************************************************************
 *
 * Prototypes for functions used for parsing a never claim.
 *
 *****************************************************************************/

static void yyerror(char*);                         /* Function to be called
						     * in case of a parse
						     * error.
						     */

int yylex();                                        /* Reads the next token
						     * from the input (this
						     * function is provided
						     * by the lexer).
						     */
%}



/******************************************************************************
 *
 * Declarations for terminal and nonterminal symbols used in the grammar rules
 * below.
 *
 *****************************************************************************/

/* Data types. */

%union {
  string* pf;
  string* str;
}

/* Keywords. */

%token NEVER IF FI GOTO SKIP

/* State labels. */

%token <str> LABEL

/* Punctuation symbols. */

%token COLON SEMICOLON DOUBLE_COLON LBRACE RBRACE LPAREN RPAREN RIGHT_ARROW

/* Propositional formulae. */

%token <pf> PROPOSITION TRUE FALSE
%right OR
%right AND
%nonassoc NOT

%type <pf> formula

%%



/******************************************************************************
 *
 * Grammar rule definitions.
 *
 *****************************************************************************/

never_claim:         NEVER LBRACE states RBRACE
                   ;

states:              state
                   | states state
                   ;

state:                 {
                         automaton->addNewState();
                       }
                     state_labels state_body
                   ;

state_labels:        LABEL COLON
                       {
                         automaton->addNewLabel(*$1);
                         delete $1;
                       }
                   | state_labels LABEL COLON
                       {
                         automaton->addNewLabel(*$2);
                         delete $2;
                       }
                   ;

state_body:          IF transitions FI SEMICOLON
                   | SKIP
                   | FALSE SEMICOLON
                       {
                         automaton->currentState()->accepting() = false;
                       }
                   | IF DOUBLE_COLON FALSE FI SEMICOLON
                       {
			 automaton->currentState()->accepting() = false;
		       }
		   ;

transitions:         transition
                   | transitions transition
                   ;

transition:          DOUBLE_COLON formula RIGHT_ARROW GOTO LABEL
                       {
                         automaton->currentState()->addTransition(*$5, $2);
                         delete $5;
                       }
                   ;

formula:             PROPOSITION
                       { 
                         $$ = $1; 
                       }
                   | TRUE
                       { 
                         $$ = $1; 
                       }
                   | FALSE
                       {
                         $$ = $1;
		       }
                   | NOT formula
                       {
                         $$ = new string;
                         *$$ += FormulaString::FormulaString::LTL_NOT;
                         *$$ += ' ';
                         *$$ += *$2;
                         delete $2;
                       }
                   | formula AND formula
                       {
                         $$ = new string;
                         *$$ += FormulaString::FormulaString::LTL_AND;
                         *$$ += ' ';
                         *$$ += *$1;
                         *$$ += ' ';
                         *$$ += *$3;
                         delete $1;
                         delete $3;
                       }
                   | formula OR formula
                       {
                         $$ = new string;
                         *$$ += FormulaString::FormulaString::LTL_OR;
                         *$$ += ' ';
                         *$$ += *$1;
                         *$$ += ' ';
                         *$$ += *$3;
                         delete $1;
                         delete $3;
                       }
                   | LPAREN formula RPAREN
                       {
                         $$ = $2;
                       }
                   ;

%%



#include "NeverClaimParser.yy.cc"

/******************************************************************************
 *
 * (Dummy) function to be called in case of a parse error.
 *
 *****************************************************************************/

void yyerror(char*)
{
  return;
}
