/*
 *  Copyright (C) 1999, 2000  Heikki Tauriainen <Heikki.Tauriainen@hut.fi>
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _spin_lbt_h_
#define _spin_lbt_h_

#include <csignal>
#include <cstdio>
#include <cstdlib>
#include <fstream>
#include <string>
#include "Exception.h"
#include "FormulaString.h"
#include "NeverClaimParser.h"

using namespace std;

namespace spin_buchi
{

using FormulaString::FormulaString;

char SPIN_TRUE[]        = "true";
char SPIN_FALSE[]       = "false";
char SPIN_NOT[]         = "!";
char SPIN_NEXT[]        = "X";
char SPIN_FINALLY[]     = "<>";
char SPIN_GLOBALLY[]    = "[]";
char SPIN_AND[]         = "&&";
char SPIN_OR[]          = "||";
char SPIN_IMPLICATION[] = "->";
char SPIN_EQUIVALENCE[] = "<->";
char SPIN_UNTIL[]       = "U";
char SPIN_V[]           = "V";

class TranslateAtom
{
public:
  static void eval(FormulaString::TraversalInfo<string*>&, const string&);
};

template<const char* C>
class TranslateConstant
{
public:
  static void eval(FormulaString::TraversalInfo<string*>&);
};

template<const char* C>
class TranslateUnary
{
public:
  static void eval(FormulaString::TraversalInfo<string*>&);
};

template<const char* C>
class TranslateBinary
{
public:
  static void eval(FormulaString::TraversalInfo<string*>&);
};



inline void TranslateAtom::eval
  (FormulaString::TraversalInfo<string*>& info, const string& atom)
{
  *(info.data) += atom;
}

template<const char* C>
inline void TranslateConstant<C>::eval
  (FormulaString::TraversalInfo<string*>& info)
{
  *(info.data) += C;
}

template<const char* C>
inline void TranslateUnary<C>::eval
  (FormulaString::TraversalInfo<string*>& info)
{
  if (info.operand == 0)
  {
    *(info.data) += C;
    *(info.data) += ' ';
  }
}

template<const char* C>
inline void TranslateBinary<C>::eval
  (FormulaString::TraversalInfo<string*>& info)
{
  switch (info.operand)
  {
  case 0 : *(info.data) += '('; break;
  case 1 : *(info.data) += ' '; *(info.data) += C; *(info.data) += ' ';
    break;
  default : *(info.data) += ')'; break;
  }
}

}

#endif  /* _spin_lbt_h_ */
